@@ -50,7 +50,6 @@
 
 #include "system.h"
 #include "argmatch.h"
-#include "dirname.h"
 #include "error.h"
 #include "lstat.h"
 #include "quote.h"
@@ -73,6 +72,7 @@ char *program_name;
 enum
 {
   INTERACTIVE_OPTION = CHAR_MAX + 1,
+  ONE_FILE_SYSTEM,
   NO_PRESERVE_ROOT,
   PRESERVE_ROOT,
   PRESUME_INPUT_TTY_OPTION
@@ -91,6 +91,7 @@ static struct option const long_opts[] =
   {"force", no_argument, NULL, 'f'},
   {"interactive", optional_argument, NULL, INTERACTIVE_OPTION},
 
+  {"one-file-system", no_argument, NULL, ONE_FILE_SYSTEM},
   {"no-preserve-root", no_argument, NULL, NO_PRESERVE_ROOT},
   {"preserve-root", no_argument, NULL, PRESERVE_ROOT},
 
@@ -171,8 +172,13 @@ Remove (unlink) the FILE(s).\n\
                           always (-i).  Without WHEN, prompt always\n\
 "), stdout);
       fputs (_("\
-      --no-preserve-root  do not treat `/' specially (the default)\n\
-      --preserve-root   fail to operate recursively on `/'\n\
+      --one-file-system  when removing a hierarchy recursively, skip any\n\
+                          directory that is on a file system different from\n\
+                          that of the corresponding command line argument\n\
+"), stdout);
+      fputs (_("\
+      --no-preserve-root  do not treat `/' specially\n\
+      --preserve-root   do not remove `/' (default)\n\
   -r, -R, --recursive   remove directories and their contents recursively\n\
   -v, --verbose         explain what is being done\n\
 "), stdout);
@@ -208,6 +214,7 @@ rm_option_init (struct rm_options *x)
 {
   x->ignore_missing_files = false;
   x->interactive = false;
+  x->one_file_system = false;
   x->recursive = false;
   x->root_dev_ino = NULL;
   x->stdin_tty = isatty (STDIN_FILENO);
@@ -221,7 +228,7 @@ rm_option_init (struct rm_options *x)
 int
 main (int argc, char **argv)
 {
-  bool preserve_root = false;
+  bool preserve_root = true;
   struct rm_options x;
   bool prompt_once = false;
   int c;
@@ -300,6 +307,10 @@ main (int argc, char **argv)
 	    break;
 	  }
 
+	case ONE_FILE_SYSTEM:
+	  x.one_file_system = true;
+	  break;
+
 	case NO_PRESERVE_ROOT:
 	  preserve_root = false;
 	  break;
@@ -344,22 +355,20 @@ main (int argc, char **argv)
 	       quote ("/"));
     }
 
-  {
-    size_t n_files = argc - optind;
-    char const *const *file = (char const *const *) argv + optind;
+  size_t n_files = argc - optind;
+  char const *const *file = (char const *const *) argv + optind;
 
-    if (prompt_once && (x.recursive || 3 < n_files))
-      {
-	fprintf (stderr,
-		 (x.recursive
-		  ? _("%s: remove all arguments recursively? ")
-		  : _("%s: remove all arguments? ")),
-		 program_name);
-	if (!yesno ())
-	  exit (EXIT_SUCCESS);
-      }
-    enum RM_status status = rm (n_files, file, &x);
-    assert (VALID_STATUS (status));
-    exit (status == RM_ERROR ? EXIT_FAILURE : EXIT_SUCCESS);
-  }
+  if (prompt_once && (x.recursive || 3 < n_files))
+    {
+      fprintf (stderr,
+	       (x.recursive
+		? _("%s: remove all arguments recursively? ")
+		: _("%s: remove all arguments? ")),
+	       program_name);
+      if (!yesno ())
+	exit (EXIT_SUCCESS);
+    }
+  enum RM_status status = rm (n_files, file, &x);
+  assert (VALID_STATUS (status));
+  exit (status == RM_ERROR ? EXIT_FAILURE : EXIT_SUCCESS);
 }
